"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.paginate = exports.Paginator = void 0;
const TAG = "knex-paginate";
class Paginator {
    constructor(query, config) {
        this.initialQuery = query.clone();
        if (typeof config === "string") {
            this.state = JSON.parse(config);
            return;
        }
        const splitColumnDetails = (column) => {
            if (!column)
                return [null, null];
            // If using a column alias
            const splitAlias = column.split(/[ ]+[aA][sS][ ]+/);
            if (splitAlias.length === 2)
                return splitAlias;
            // If using a table alias
            const columnWithTableAlias = column.split(".");
            if (columnWithTableAlias.length === 2) {
                return [column, columnWithTableAlias[1]];
            }
            // If only using a column name
            return [column, column];
        };
        const [orderByColumn, orderByColumnName] = splitColumnDetails(config.orderByColumn);
        const [cursorColumn, cursorColumnName] = splitColumnDetails(config.cursorColumn);
        this.state = Object.assign(Object.assign({}, config), { orderByColumnName,
            orderByColumn,
            cursorColumnName,
            cursorColumn });
    }
    exec(opts = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const { previous } = opts;
            const { order, cursorColumn, cursorColumnName, orderByColumnName } = this.state;
            const paginateConfig = previous
                ? Object.assign(Object.assign({}, this.state), { order: order === "asc" ? "desc" : "asc", cursor: this.state.tailCursor, orderByValue: this.state.tailOrderByValue }) : this.state;
            const results = yield paginate(this.initialQuery.clone(), Object.assign(Object.assign({}, paginateConfig), { pageOffset: opts.pageOffset }));
            if (!(results === null || results === void 0 ? void 0 : results.length))
                return [];
            if (previous) {
                results.sort((a, b) => {
                    const [first, second] = order === "asc" ? [a, b] : [b, a];
                    if (orderByColumnName) {
                        if (first[orderByColumnName] < second[orderByColumnName]) {
                            return -1;
                        }
                        if (first[orderByColumnName] > second[orderByColumnName]) {
                            return 1;
                        }
                    }
                    if (first[cursorColumnName] < second[cursorColumnName]) {
                        return -1;
                    }
                    if (first[cursorColumnName] > second[cursorColumnName]) {
                        return 1;
                    }
                    console.warn(`${TAG}: ${cursorColumn} is not unique, cursorColumn must be unique to page correctly.`);
                    return 0;
                });
            }
            const length = results.length;
            Object.assign(this.state, {
                cursor: results[length - 1][cursorColumnName],
                orderByValue: results[length - 1][orderByColumnName],
                tailCursor: results[0][cursorColumnName],
                tailOrderByValue: results[0][orderByColumnName],
            });
            return results;
        });
    }
    /** get the next page of results */
    next(options = {}) {
        return this.exec(options);
    }
    /** get the previous page of results */
    previous(options = {}) {
        return this.exec(Object.assign(Object.assign({}, options), { previous: true }));
    }
    get value() {
        return Object.assign({}, this.state);
    }
    toJSON() {
        return this.value;
    }
}
exports.Paginator = Paginator;
function paginate(query, config) {
    const { cursor, cursorColumn, order, orderByColumn, pageSize, orderByValue, pageOffset, } = config;
    return query
        .modify((_qb) => {
        if (cursor == null && orderByValue == null)
            return;
        if (cursor == null) {
            throw new Error(`${TAG}: no cursor provided (${cursor})`);
        }
        if (orderByValue == null) {
            _qb.whereRaw(`${cursorColumn} ${order === "asc" ? ">" : "<"} ?`, cursor);
        }
        else {
            _qb
                .whereRaw(`${orderByColumn} ${order === "asc" ? ">=" : "<="} ?`, orderByValue)
                .andWhereNot((_andWhereNot) => {
                _andWhereNot
                    .whereRaw(`${orderByColumn} = ?`, orderByValue)
                    .andWhereRaw(`${cursorColumn} ${order === "asc" ? "<=" : ">="} ?`, cursor);
            });
        }
    })
        .orderByRaw(`${orderByColumn ? `${orderByColumn} ${order}, ` : ""}${cursorColumn} ${order}`)
        .limit(pageSize)
        .modify((_qb) => {
        if (pageOffset) {
            _qb.offset(pageOffset * pageSize);
        }
    });
}
exports.paginate = paginate;
//# sourceMappingURL=index.js.map